#include "common.h"
#include "common_transform.h"
#include "config.h"
#include <time.h>

#ifndef MHMESH_CLASS
#define MHMESH_CLASS

/** 
@brief Does all the Manifold Harmonics calculations.
*/
class MHMesh
{
public:
	Mesh* mesh;					/**< mesh datastructure from OpenMesh */
	MatrixXd mhb;				/**< Manifold Harmonics Basis */
	VectorXd eigenvalues;		/**< the eigenvalues corresponding to the vectors of the MHB*/

	VectorXd filter;			/**< buffer for the filter function that is applied to the harmonics */

	VectorXd mhtX;				/**< x coordinates of the Manifold Harmonics Transformation */
	VectorXd mhtY;				/**< y coordinates of the Manifold Harmonics Transformation */
	VectorXd mhtZ;				/**< z coordinates of the Manifold Harmonics Transformation */
	VectorXd reconstructedX;	/**< x coordinates of the reconstructed mesh */
	VectorXd reconstructedY;	/**< x coordinates of the reconstructed mesh */
	VectorXd reconstructedZ;	/**< x coordinates of the reconstructed mesh */
	double* reconstructedData;	/**< x coordinates of the reconstructed mesh */

	/// default constructor
	MHMesh() {
		mhbcalced=false;
		mhtcalced=false;
		reconstructed=false;
		filter = VectorXd(Config::startFreqs);
		filter.fill(1);
	}
	
	/// computes Manifold Harmonics Basis
	bool calcMHB();
	
	/// computes Manifold Harmonics Transformation
	void calcMHT();
	
	/** Reconstructs the mesh.
	@param numEigVecs the number of most significant harmonics that are used for reconstruction
	*/
	bool reconstruct(int numEigVecs);
	
	/// only for debugging purposes
	void writeToFiles();

	/// returns the position of reconstructed vertex of index ind
	float *getVert(int ind);

	/** returns an array of the positions of all reconstructed vertices.
	The array contains x,y,z coordinates of first point, then x,y,z of second point and so on.
	Just as needed to feed openGL.
	*/
	float *getPositions();

	OpenMesh::VPropHandleT<int> index;			/**< provides access to the index of a vertex */
	OpenMesh::VPropHandleT<float> vtarea;		/**< provides access to the voronoi area of a vertex */

protected:
	bool mhbcalced;				/**< indicates if Manifold Harmonics Basis has already been computed */
	bool mhtcalced;				/**< indicates if Manifold Harmonics Transformation has already been computed */
	bool reconstructed;			/**< indicates if reconstruction has already been computed */
	int numberofvertices;		/**< stores the mesh's number of vertices. actually useless. */
	int numEigVecs;				/**< duplicate of numberofvertices. actually useless and very stupid. */
};

#endif