#include "audio.h"

Audio::Audio(){
	FMOD::System_Create(&system);
	system->init(32, FMOD_INIT_NORMAL, 0);
	numFreqs = Config::startFreqs;
	left = new float[Config::startFreqs];
	right = new float[Config::startFreqs];
	eq = new float[Config::startFreqs];
	low=0.5;
	mid=2.0;
	high=10.0;
	ready=false;
}

bool Audio::loadSong(const char *path){
	channel->stop();
	FMOD_RESULT result = system->createSound(path, FMOD_DEFAULT, 0, &sound);
	if(result==FMOD_OK){
		system->playSound(FMOD_CHANNEL_FREE, sound, false, &channel);
		cout << "Audio: Loaded " << path << endl;
		updateEQ(FREQS,0);
		ready=true;
		return true;
	}

	return false;
}

void Audio::rePlay(){
	if(!ready)
		return;
	channel->stop();
	system->playSound(FMOD_CHANNEL_FREE, sound, false, &channel);
}

bool Audio::playPause(){
	bool paused;
	channel->getPaused(&paused);
	if(paused){
		channel->setPaused(false);
		return true;
	} else {
		channel->setPaused(true);
		return false;
	}
}

void Audio::getFilter(VectorXd *filter){

	if(ready){
		system->getSpectrum(left, numFreqs, 0, FMOD_DSP_FFT_WINDOW_BLACKMAN);
		system->getSpectrum(right, numFreqs, 1, FMOD_DSP_FFT_WINDOW_BLACKMAN);
		
		float newVal=0;	//buffer
		
		for(int i=0; i<numFreqs; i++){
			newVal = 0.8 + (left[i]+right[i]) * (i+1) * eq[i];
			(*filter)[i] += (newVal - (*filter)[i]) / Config::smoothing;
		}
	} else {
		for(int i=0; i<numFreqs; i++){
			(*filter)[i] = 1;
		}
	}
}

void Audio::updateEQ(EQType t, int v){

	if(t==LOW) low=(float)v/200;
	else if(t==MID) mid=(float)v/50;
	else if(t==HIGH) high=(float)v/10;


	//create eq: 
	//in interval [0, numFreqs/4-1] (first fourth) interpolate between low and mid;
	//in interval [numFreqs/4, numFreqs-1] (rest) interpolate between mid and high.
	for(int i=0; i<numFreqs/4; i++){
		eq[i]= low + (mid-low)/(numFreqs/2) * i;
	}
	for(int i=0; i<numFreqs/4*3; i++){
		eq[i+numFreqs/4]= mid + (high-mid)/(numFreqs/2) * i;
	}
}


Audio::~Audio(){
	system->release();
	delete left;
	delete right;
}